/****************************************************************************
 *   Copyright (C) 2012 by Claas Anders "CaScAdE" Rathje                    *
 *   admiralcascade@gmail.com                                               *
 *   Project-URL: http://www.mylifesucks.de/oss/c-scale/                    *
 *                                                                          *
 *   This program is free software; you can redistribute it and/or modify   *
 *   it under the terms of the GNU General Public License as published by   *
 *   the Free Software Foundation; either version 2 of the License.         *
 *                                                                          *
 *   This program is distributed in the hope that it will be useful,        *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of         *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for more details.                           *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to the                          *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include "uart.h"

#define BAUD_RATE 19200

volatile uint8_t received = 0;

volatile uint8_t txd0_buffer[TXD0_BUFFER_LEN];
volatile uint8_t *pTxData0 = 0;
volatile uint8_t TxData0Len = 0;

/**
 * init uart
 */
void uart_init() {
    UBRRL = (F_CPU / (16UL * BAUD_RATE)) - 1;

    // Enable receiver and transmitter; enable RX interrupt
    UCSRB = (1 << RXEN) | (1 << TXEN) | (1 << RXCIE) | (1 << TXCIE);

    //asynchronous 8N1
    UCSRC = (1 << URSEL) | (3 << UCSZ0);
}

/**
 * send a single <character> through uart
 */
void uart_putc(unsigned char character) {
    // wait until UDR ready
    while (!(UCSRA & (1 << UDRE)));
    UDR = character;
}

/**
 * send a <string> throught uart
 */
void uart_puts(char *s) {
    while (*s) {
        uart_putc(*s);
        s++;
    }
}

/**
 * send a <string> from pgm space throught uart
 */
void uart_puts_pgm(char *string) {
 	while (pgm_read_byte(string) != 0x00)
		uart_putc(pgm_read_byte(string++));
}


ISR(USART_TXC_vect) {
	if(TxData0Len > 0 && TxData0Len != TXD0_BLOCKED) {
		UDR = *pTxData0++;		
		TxData0Len--;
	}
}

ISR(USART_RXC_vect) {
	uint8_t c;
 	// catch the received byte
    c = UDR;
	received = c;
}	
